//original work by Jeff Fay
//some optimisations by bartavelle@bandecon.com

#include <stdio.h>
#include <string.h>
#include "misc.h"
#include "formats.h"
#include "common.h"
#include "domino_md.h"

#ifndef DOMINO_KEYS
# define DOMINO_KEYS 1
#endif

/*preprocessor constants that John The Ripper likes*/
#define FORMAT_LABEL                   "domino"
#define FORMAT_NAME                    "Domino 4"
#define ALGORITHM_NAME			"Domino 4 (RSA MD)"
#define BENCHMARK_COMMENT              ""
#define BENCHMARK_LENGTH               -1
#define PLAINTEXT_LENGTH               64
#define CIPHERTEXT_LENGTH              32
#define BINARY_SIZE                    16
#define SALT_SIZE                      0
#define MIN_KEYS_PER_CRYPT             DOMINO_KEYS
#define MAX_KEYS_PER_CRYPT             DOMINO_KEYS

/*A struct used for JTR's benchmarks*/
static struct fmt_tests tests[] = {
  {"06E0A50B579AD2CD5FFDC48564627EE7", "secret"},
  {"(355E98E7C7B59BD810ED845AD0FD2FC4)", "password"},
  {"CD2D90E8E00D8A2A63A81F531EA8A9A3", "lotus"},
  {"(69D90B46B1AC0912E5CCF858094BBBFC)", "dirtydog"},
  {NULL}
};

/*Some more JTR variables*/
static char crypt_key[BINARY_SIZE*MAX_KEYS_PER_CRYPT]; /* TODO buffer overflow to correct */
static char saved_key[PLAINTEXT_LENGTH*MAX_KEYS_PER_CRYPT + 1];
static unsigned int key_length[MAX_KEYS_PER_CRYPT];

/*Utility function to convert hex to bin */
static void * binary (char *ciphertext)
{
  static char realcipher[BINARY_SIZE];
  int i;
  int m;

  i = 0;
  if(strlen(ciphertext)==CIPHERTEXT_LENGTH+2)
	  m = 1;
  else
	  m = 0;
  for (i = 0; i < BINARY_SIZE; i++)
      realcipher[i] = atoi16[ARCH_INDEX(ciphertext[i*2+m])]*16 + atoi16[ARCH_INDEX(ciphertext[i*2+1+m])];
  return ((void *) realcipher);
}

/*Another function required by JTR: decides whether we have a valid
 * ciphertext */
static int valid (char *ciphertext)
{
  int i;
  int m;

  if(strlen(ciphertext)==CIPHERTEXT_LENGTH+2)
	  m = 1;
  else
	  m = 0;
  
  for (i = m; i < CIPHERTEXT_LENGTH + m; i++)
	  if (!(((ciphertext[i] >= '0') && (ciphertext[i] <= '9'))
				  || ((ciphertext[i] >= 'a') && (ciphertext[i] <= 'f'))
				  || ((ciphertext[i] >= 'A') && (ciphertext[i] <= 'F'))))
	  {
		  return 0;
	  }
  return 1;
}

/*sets the value of saved_key so we can play with it*/
static void set_key (char *key, int index)
{
	unsigned char x;


	x=0;
	while((*key) && (x<15) )
	{
		saved_key[ARCH_INDEX(x)+index*PLAINTEXT_LENGTH] = *key;
		key++;
		x++;
	}
	memset (saved_key + x + index*PLAINTEXT_LENGTH, 16 - x, 16 - x);
	key_length[index] = x+1;
}
		
/*retrieves the saved key; used by JTR*/
static char * get_key (int index)
{
	static char * out2[PLAINTEXT_LENGTH];
	strnzcpy((char *)out2, saved_key + PLAINTEXT_LENGTH*index, key_length[index]);
	return (char *)out2;
}

static int cmp_all (void *binary, int index)
{	
	int i = 0;
	while(i<BINARY_SIZE/sizeof(int))
	{
#ifdef LOTUS_ASM 
		if (
				(((int *)binary)[i]!=((int *)crypt_key)[i]) &&
				(((int *)binary)[i]!=((int *)crypt_key)[i+BINARY_SIZE/sizeof(int)]) &&
				(((int *)binary)[i]!=((int *)crypt_key)[i+2*BINARY_SIZE/sizeof(int)]) 
		)
			return 0;
#else
		if(((int *)binary)[i]!=((int *)crypt_key)[i])
			return 0;
#endif
		i++;
	}
	return 1;
}

static int cmp_one (void *binary, int index)
{	
	int i = 0;
	while(i<BINARY_SIZE/sizeof(int))
	{
#ifdef LOTUS_ASM 
		if (
				(((int *)binary)[i]!=((int *)crypt_key)[i+index*BINARY_SIZE/sizeof(int)]) 
		)
			return 0;
#else
		if(((int *)binary)[i]!=((int *)crypt_key)[i])
			return 0;
#endif
		i++;
	}
	return 1;
}


static int cmp_exact (char *source, int index) 
{
	return 1;
}

static void set_salt (void *salt) { }

/*Beginning of private functions*/
/* Takes the plaintext password and generates the second row of our
 * working matrix for the final call to the mixing function*/

/*the last public function; generates ciphertext*/
static void crypt_all (int count)
{
	domino_md((unsigned char*)saved_key, key_length, (unsigned char*)crypt_key);
}

static int get_hash1(int index) { return (((unsigned int *)crypt_key)[ARCH_INDEX(index*BINARY_SIZE/sizeof(int))] & 0xf); }
static int get_hash2(int index) { return (((unsigned int *)crypt_key)[ARCH_INDEX(index*BINARY_SIZE/sizeof(int))] & 0xff); }
static int get_hash3(int index) { return (((unsigned int *)crypt_key)[ARCH_INDEX(index*BINARY_SIZE/sizeof(int))] & 0xfff); }
static int binary_hash1(void * binary) { return (((unsigned int *)binary)[0] & 0xf); }
static int binary_hash2(void * binary) { return (((unsigned int *)binary)[0] & 0xff); }
static int binary_hash3(void * binary) { return (((unsigned int *)binary)[0] & 0xfff); }

/* C's version of a class specifier */
struct fmt_main fmt_lotus5 = {
	{
		FORMAT_LABEL,
		FORMAT_NAME,
		ALGORITHM_NAME,
		BENCHMARK_COMMENT,
		BENCHMARK_LENGTH,
		PLAINTEXT_LENGTH,
		BINARY_SIZE,
		SALT_SIZE,
		MIN_KEYS_PER_CRYPT,
		MAX_KEYS_PER_CRYPT,
		FMT_CASE | FMT_8_BIT,
		tests
	}, {
		fmt_default_init,
		valid,
		fmt_default_split,
		binary,
		fmt_default_salt,
		{
			binary_hash1,
                        binary_hash2,
                        binary_hash3
		},
		fmt_default_salt_hash,
		set_salt,
		set_key,
		get_key,
		fmt_default_clear_keys,
		crypt_all,
		{
			get_hash1,
                        get_hash2,
                        get_hash3
		},
		cmp_all,
		cmp_one,
		cmp_exact}
};
