#include <string.h>

#include "arch.h"
#include "misc.h"
#include "common.h"
#include "formats.h"
#include "sha.h"


#define FORMAT_LABEL			"mysqlsha1"
#define FORMAT_NAME			"MySQL SHA1"
#ifdef MMX_COEF
#if (MMX_COEF == 2)
#define ALGORITHM_NAME			"mysql-sha1 MMX"
#else
#define ALGORITHM_NAME			"mysql-sha1 SSE2"
#endif
#else
#define ALGORITHM_NAME			"mysql-sha1"
#endif

#ifdef MMX_TYPE
#define BENCHMARK_COMMENT		MMX_TYPE
#else
#define BENCHMARK_COMMENT		""
#endif
#define BENCHMARK_LENGTH		-1

#define PLAINTEXT_LENGTH		32
#define CIPHERTEXT_LENGTH		41

#define BINARY_SIZE			20
#define SALT_SIZE			0

#ifdef MMX_COEF
#define MIN_KEYS_PER_CRYPT		MMX_COEF
#define MAX_KEYS_PER_CRYPT		MMX_COEF
//#define GETPOS(i, index)		( (index)*4 + (i& (0xffffffff-3) )*MMX_COEF + ((i)&3) ) //std getpos
#define GETPOS(i, index)		( (index)*4 + (i& (0xffffffff-3) )*MMX_COEF + (3-((i)&3)) ) //for endianity conversion
#else
#define MIN_KEYS_PER_CRYPT		1
#define MAX_KEYS_PER_CRYPT		1
#endif

static struct fmt_tests mysqlsha1_tests[] = {
    {"*5AD8F88516BD021DD43F171E2C785C69F8E54ADB", "tere"},
    {"*2C905879F74F28F8570989947D06A8429FB943E6", "verysecretpassword"},
    {"*A8A397146B1A5F8C8CF26404668EFD762A1B7B82", "________________________________"},
    {"*F9F1470004E888963FB466A5452C9CBD9DF6239C", "12345678123456781234567812345678"},
    {"*97CF7A3ACBE0CA58D5391AC8377B5D9AC11D46D9", "' OR 1 /*'"},
    {"*2470C0C06DEE42FD1618BB99005ADCA2EC9D1E19", "password"},
    {"*7534F9EAEE5B69A586D1E9C1ACE3E3F9F6FCC446", "5"},
	{NULL}
};

#ifdef MMX_COEF
static char saved_key[80*MMX_COEF*4 + 1] __attribute__ ((aligned(16)));
static char tmp_key[80*MMX_COEF*4 + 1] __attribute__ ((aligned(16)));
static char crypt_key[BINARY_SIZE*MMX_COEF+1] __attribute__ ((aligned(16)));
unsigned char out[PLAINTEXT_LENGTH];
#else
static char saved_key[PLAINTEXT_LENGTH + 1];
static char crypt_key[BINARY_SIZE+1];
static SHA_CTX ctx;
#endif

static int valid(char *ciphertext)
{
	int i;

	if (strlen(ciphertext) != CIPHERTEXT_LENGTH) return 0;
	if (ciphertext[0]!='*')
		return 0;
	for (i = 1; i < CIPHERTEXT_LENGTH; i++){
		if (!(  (('0' <= ciphertext[i])&&(ciphertext[i] <= '9')) ||
					(('a' <= ciphertext[i])&&(ciphertext[i] <= 'f'))  
					|| (('A' <= ciphertext[i])&&(ciphertext[i] <= 'F'))))
			return 0;
	}
	return 1;
}

static void mysqlsha1_set_salt(void *salt) { }

static void mysqlsha1_init(void)
{
#ifdef MMX_COEF
	memset(saved_key, 0, sizeof(saved_key));
#endif
}

static void mysqlsha1_set_key(char *key, int index)
{
#ifdef MMX_COEF
	int len;
	int i;
	
	if(index==0)
	{
		memset(saved_key, 0, sizeof(saved_key));
	}
	len = strlen(key);
	if(len>PLAINTEXT_LENGTH)
		len = PLAINTEXT_LENGTH;

	for(i=0;i<len;i++)
		saved_key[GETPOS(i, index)] = key[i];

	saved_key[GETPOS(i, index)] = 0x80;
	((unsigned int *)saved_key)[15*MMX_COEF+index] = (len<<3);
#else
	strnzcpy(saved_key, key, PLAINTEXT_LENGTH+1);
#endif
}

static char *mysqlsha1_get_key(int index) {
#ifdef MMX_COEF
	unsigned int i,s;

	s = ((unsigned int *)saved_key)[15*MMX_COEF+index] >> 3;
	
	for(i=0;i<s;i++)
		out[i] = saved_key[ GETPOS(i, index) ];
	out[i] = 0;
	return (char*)out;
#else
	return saved_key;
#endif
}

static int mysqlsha1_cmp_all(void *binary, int index) { 
	int i=0;
#ifdef MMX_COEF
	while(i< (BINARY_SIZE/4) )
	{
		if (
			( ((unsigned long *)binary)[i] != ((unsigned long *)crypt_key)[i*MMX_COEF])
			&& ( ((unsigned long *)binary)[i] != ((unsigned long *)crypt_key)[i*MMX_COEF+1])
#if (MMX_COEF > 3)
			&& ( ((unsigned long *)binary)[i] != ((unsigned long *)crypt_key)[i*MMX_COEF+2])
			&& ( ((unsigned long *)binary)[i] != ((unsigned long *)crypt_key)[i*MMX_COEF+3])
#endif
		)
			return 0;
		i++;
	}
#else
	while(i<BINARY_SIZE)
	{
		if(((char *)binary)[i]!=((char *)crypt_key)[i])
			return 0;
		i++;
	}
#endif
	return 1;
}

static int mysqlsha1_cmp_exact(char *source, int count){
  return (1);
}

static int mysqlsha1_cmp_one(void * binary, int index)
{
#ifdef MMX_COEF
	int i = 0;
	for(i=0;i<(BINARY_SIZE/4);i++)
		if ( ((unsigned long *)binary)[i] != ((unsigned long *)crypt_key)[i*MMX_COEF+index] )
			return 0;
	return 1;
#else
	return mysqlsha1_cmp_all(binary, index);
#endif
}

static void mysqlsha1_crypt_all(int count) {  
  // get plaintext input in saved_key put it into ciphertext crypt_key
#ifdef MMX_COEF
	int i;

	shammx((unsigned char*)crypt_key, (unsigned char*)saved_key);
	memcpy(tmp_key, crypt_key, BINARY_SIZE*MMX_COEF);
	memset(tmp_key+BINARY_SIZE*MMX_COEF, 0, (80*4-BINARY_SIZE)*MMX_COEF);
	for(i=0;i<MMX_COEF;i++)
	{
		tmp_key[GETPOS(20, i)] = 0x80;
		((unsigned int *)tmp_key)[15*MMX_COEF+i] = (20<<3);
	}
	shammx((unsigned char*)crypt_key, (unsigned char*)tmp_key);
#else
	SHA1_Init( &ctx );
	SHA1_Update( &ctx, saved_key, strlen( saved_key ) );
	SHA1_Final( crypt_key, &ctx);
#endif
  
}

static void * mysqlsha1_binary(char *ciphertext) 
{
	static char realcipher[BINARY_SIZE];
	int i;

	for(i=0;i<BINARY_SIZE;i++)
	{
		realcipher[i] = atoi16[ARCH_INDEX(ciphertext[i*2+1])]*16 + atoi16[ARCH_INDEX(ciphertext[i*2+2])];
	}
#ifdef MMX_COEF
	alter_endianity((unsigned char*)realcipher, BINARY_SIZE);
#endif
	return (void *)realcipher;
}

static int binary_hash_0(void *binary)
{
  return ((unsigned int *)binary)[0] & 0xF;
}

static int binary_hash_1(void *binary)
{
  return ((unsigned int *)binary)[0] & 0xFF;
}

static int binary_hash_2(void *binary)
{
  return ((unsigned int *)binary)[0] & 0xFFF;
}

static int get_hash_0(int index)
{
  return ((unsigned long *)crypt_key)[index] & 0xF;
}

static int get_hash_1(int index)
{
  return ((unsigned long *)crypt_key)[index] & 0xFF;
}

static int get_hash_2(int index)
{
  return ((unsigned long *)crypt_key)[index] & 0xFFF;
}

struct fmt_main fmt_mysqlsha1 = {
	{
		FORMAT_LABEL,
		FORMAT_NAME,
		ALGORITHM_NAME,
		BENCHMARK_COMMENT,
		BENCHMARK_LENGTH,
		PLAINTEXT_LENGTH,
		BINARY_SIZE,
		SALT_SIZE,
		MIN_KEYS_PER_CRYPT,
		MAX_KEYS_PER_CRYPT,
		FMT_CASE | FMT_8_BIT,
		mysqlsha1_tests
	}, {
		mysqlsha1_init,
		valid,
		fmt_default_split,
		mysqlsha1_binary,
		fmt_default_salt,
		{
			binary_hash_0,
			binary_hash_1,
			binary_hash_2
		},
		fmt_default_salt_hash,
		mysqlsha1_set_salt,
		mysqlsha1_set_key,
		mysqlsha1_get_key,
		fmt_default_clear_keys,
		mysqlsha1_crypt_all,
		{
			get_hash_0,
			get_hash_1,
			get_hash_2
		},
		mysqlsha1_cmp_all,
		mysqlsha1_cmp_one,
		mysqlsha1_cmp_exact
	}
};
