
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

// johntr includes
#include "arch.h"
#include "misc.h"
#include "formats.h"
#include "common.h"
#include "des.h"

//johntr defines
#define FORMAT_LABEL "vnc"
#define FORMAT_NAME "Tight VNC"
#define ALGORITHM_NAME "DES"

#define BENCHMARK_COMMENT ""
#define BENCHMARK_LENGTH -1

#define PLAINTEXT_LENGTH 32

#define CIPHERTEXT_LENGTH 16

#define BINARY_SIZE 8
#define SALT_SIZE 0

#define MIN_KEYS_PER_CRYPT 1
#define MAX_KEYS_PER_CRYPT 1

//test cases
static struct fmt_tests vnc_tests[] = {
  {"5ca3f12ad8904f75", "euler"},
  {NULL}
};

static DES_key_schedule desschedule;

//stores the ciphertext for value currently being tested
static char crypt_key[BINARY_SIZE+1];

//used by set_key
static char saved_key[PLAINTEXT_LENGTH + 1];

static void des_init(void)
{
	unsigned char deskey[8] = {23,82,107,6,35,78,88,7};
	my_des_set_key(&deskey, &desschedule);
}

static int vnc_valid(char *ciphertext) { //returns 0 for invalid ciphertexts

  int i; //used as counter in loop
  
  //ciphertext is 16 characters
  if (strlen(ciphertext) != 16) return 0;  

  //ciphertext is ASCII representation of hex digits
  for (i = 0; i < 16; i++){
    if (!(  ((48 <= ciphertext[i])&&(ciphertext[i] <= 57)) ||
	    ((97 <= ciphertext[i])&&(ciphertext[i] <= 102))  ))
      return 0;
  }

  return 1;
}

static void vnc_set_salt(void *salt) { }

static void vnc_set_key(char *key, int index) {
  strnzcpy(saved_key, key, PLAINTEXT_LENGTH+1);
}

static char *vnc_get_key(int index) {
    return saved_key;
}

static int vnc_cmp_all(void *binary, int index) { 	
	int i = 0;
	while(i<BINARY_SIZE)
	{
		if(((char *)binary)[i]!=((char *)crypt_key)[i])
			return 0;
		i++;
	}
	return 1;
}

static int vnc_cmp_exact(char *source, int count){
  return (1);
}

static void vnc_crypt_all(int count) {
	unsigned char tmpkey[8];

	memset(crypt_key, 0, 8);
	strncpy(crypt_key, saved_key, 8);
	DES_encrypt1(&crypt_key, &desschedule);
	printf("\n");
	dump_stuff(crypt_key, 8);
}

struct fmt_main fmt_tightvnc = {
  {
    FORMAT_LABEL,
    FORMAT_NAME,
    ALGORITHM_NAME,
    BENCHMARK_COMMENT,
    BENCHMARK_LENGTH,
    PLAINTEXT_LENGTH,
    BINARY_SIZE,
    SALT_SIZE,
    MIN_KEYS_PER_CRYPT,
    MAX_KEYS_PER_CRYPT,
    FMT_CASE | FMT_8_BIT,
    vnc_tests
    }, {
      fmt_default_init,
      vnc_valid, 
      fmt_default_split,
      fmt_default_binary,
      fmt_default_salt,
      {
	fmt_default_binary_hash,
	fmt_default_binary_hash,
	fmt_default_binary_hash
      },
      fmt_default_salt_hash,
      vnc_set_salt,
      vnc_set_key,
      vnc_get_key,
      fmt_default_clear_keys,
      vnc_crypt_all,
      {
	fmt_default_get_hash,
	fmt_default_get_hash,
	fmt_default_get_hash
      },
      vnc_cmp_all,
      vnc_cmp_all,
      vnc_cmp_exact
    }
};
